/**********************************************************************
  sim DISK (Pseudo DISK device)
  $Id: dev_disk.c,v 1.3 2006/03/19 07:35:44 tane Exp $
**********************************************************************/
#include "common.h"
#include "ppcsim.h"
#include "processor.h"
#include "funcs.h"
#include "memory.h"
#include "op_exec.h"
#include "sim_io.h"

int dev_disk( unsigned int port , struct MACHINE_RESOURCE *mres )
{
  int cmd ;
  int i ;
  int ret_code ;
  
  ret_code = NORMAL_EXIT ;
  
  /** access DISK device **/
  cmd  = mres->mem[seg(DISK_CMD     +3)][lad(DISK_CMD     +3)] ;
  if( cmd != DISK_NOP ){
    int drv, dtrk, dtop, dlen ;
    drv  = mres->mem[seg(DISK_DRIVE   +3)][lad(DISK_DRIVE   +3)] ;
    dtrk = (mres->mem[seg(DISK_TRACK   +0)][lad(DISK_TRACK   +0)]<<24) +
      (mres->mem[seg(DISK_TRACK   +1)][lad(DISK_TRACK   +1)]<<16) +
      (mres->mem[seg(DISK_TRACK   +2)][lad(DISK_TRACK   +2)]<< 8) +
      mres->mem[seg(DISK_TRACK   +3)][lad(DISK_TRACK   +3)] ;
    dtop = (mres->mem[seg(DISK_DATA_TOP+0)][lad(DISK_DATA_TOP+0)]<<24) +
      (mres->mem[seg(DISK_DATA_TOP+1)][lad(DISK_DATA_TOP+1)]<<16) +
      (mres->mem[seg(DISK_DATA_TOP+2)][lad(DISK_DATA_TOP+2)]<< 8) +
      mres->mem[seg(DISK_DATA_TOP+3)][lad(DISK_DATA_TOP+3)] ;
    dlen = (mres->mem[seg(DISK_DATA_LEN+0)][lad(DISK_DATA_LEN+0)]<<24) +
      (mres->mem[seg(DISK_DATA_LEN+1)][lad(DISK_DATA_LEN+1)]<<16) +
      (mres->mem[seg(DISK_DATA_LEN+2)][lad(DISK_DATA_LEN+2)]<< 8) +
      mres->mem[seg(DISK_DATA_LEN+3)][lad(DISK_DATA_LEN+3)] ;
    
    mres->mem[seg(DISK_CMD+3)][lad(DISK_CMD+3)]=DISK_NOP ;
    
    if( drv<0 || drv >= SIM_DISK_DRIVE_NUM ){
      mres->mem[seg(DISK_ACK+3)][lad(DISK_ACK+3)]=DISK_ACK_FAIL ;
    }else{
      if( simDisk[drv].mount==VALID ){
	switch( cmd ){
	case DISK_INQ :
	  for( i=0 ; i<DISK_INQ_LEN ; i++ ){
	    char *p=(char *)simDisk[drv].info ;
	    mres->mem[seg(dtop+i)][lad(dtop+i)]=p[i] ;
	  }
	  mres->mem[seg(DISK_ACK+3)][lad(DISK_ACK+3)]=DISK_ACK_IDLE ;
	  break ;
	case DISK_WRITE_INQ :
	  for( i=0 ; i<DISK_INQ_LEN ; i++ ){
	    char *p=(char *)simDisk[drv].info ;
	    p[i]=mres->mem[seg(dtop+i)][lad(dtop+i)] ;
	  }
	  mres->mem[seg(DISK_ACK+3)][lad(DISK_ACK+3)]=DISK_ACK_IDLE ;
	  break ;
	case DISK_READ :
	  if( (simDisk[drv].type & DISK_TYPE_EN_READ)==0 ){
	    mres->mem[seg(DISK_ACK+3)][lad(DISK_ACK+3)]=DISK_ACK_FAIL ;
	  }else{
	    for( i=0 ; i<dlen ; i++ ){
	      if( (dtrk*simDisk[drv].tlen+i)>(simDisk[drv].tnum * simDisk[drv].tlen) ) break ;
	      mres->mem[seg(dtop+i)][lad(dtop+i)]=simDisk[drv].buf[dtrk*simDisk[drv].tlen+i] ;
	    }
	    mres->mem[seg(DISK_ACK+3)][lad(DISK_ACK+3)]=DISK_ACK_IDLE ;
	  }
	  break ;
	case DISK_WRITE :
	  if( (simDisk[drv].type & DISK_TYPE_EN_WRITE)==0 ){
	    mres->mem[seg(DISK_ACK+3)][lad(DISK_ACK+3)]=DISK_ACK_FAIL ;
	  }else{
	    for( i=0 ; i<dlen ; i++ ){
	      if( (dtrk*simDisk[drv].tlen+i)>(simDisk[drv].tnum * simDisk[drv].tlen) ) break ;
	      simDisk[drv].buf[dtrk*simDisk[drv].tlen+i]=mres->mem[seg(dtop+i)][lad(dtop+i)] ;
	    }
	    mres->mem[seg(DISK_ACK+3)][lad(DISK_ACK+3)]=DISK_ACK_IDLE ;
	  }
	  break ;
	default :
	  SYSMSG("Sim DISK invalid access command. (0x%08x)\n",cmd) ;
	  ret_code=-1 ;
	}
      }
    }
  }
  return( ret_code ) ;
}


int dev_disk_init( struct MACHINE_RESOURCE *mres )
{
  int i ;
  
  for( i=0 ; i<SIM_DISK_DRIVE_NUM ; i++ ){
    simDisk[i].mount = INVALID ;
    simDisk[i].type  = 0 ;
    simDisk[i].tnum  = 0 ;
    simDisk[i].info  = NULL ;
    simDisk[i].buf   = NULL ;
  }

  return( NORMAL_EXIT ) ;
}
